@@ -2,8 +2,8 @@ use super::*;
 
 #[derive(Debug, PartialEq)]
 pub(crate) struct CompileError<'src> {
-  pub(crate) token: Token<'src>,
   pub(crate) kind: Box<CompileErrorKind<'src>>,
+  pub(crate) token: Token<'src>,
 }
 
 impl<'src> CompileError<'src> {
@@ -54,7 +54,7 @@ impl Display for CompileError<'_> {
         }
       }
       BacktickShebang => write!(f, "Backticks may not start with `#!`"),
-      CircularRecipeDependency { recipe, ref circle } => {
+      CircularRecipeDependency { recipe, circle } => {
         if circle.len() == 2 {
           write!(f, "Recipe `{recipe}` depends on itself")
         } else {
@@ -65,10 +65,7 @@ impl Display for CompileError<'_> {
           )
         }
       }
-      CircularVariableDependency {
-        variable,
-        ref circle,
-      } => {
+      CircularVariableDependency { variable, circle } => {
         if circle.len() == 2 {
           write!(f, "Variable `{variable}` is defined in terms of itself")
         } else {
@@ -106,6 +103,10 @@ impl Display for CompileError<'_> {
         first.ordinal(),
         self.token.line.ordinal(),
       ),
+      DuplicateDefault { recipe } => write!(
+        f,
+        "Recipe `{recipe}` has duplicate `[default]` attribute, which may only appear once per module",
+      ),
       DuplicateParameter { recipe, parameter } => {
         write!(f, "Recipe `{recipe}` has duplicate parameter `{parameter}`")
       }
@@ -165,7 +166,7 @@ impl Display for CompileError<'_> {
         ShowWhitespace(expected),
         ShowWhitespace(found)
       ),
-      Internal { ref message } => write!(
+      Internal { message } => write!(
         f,
         "Internal error, this may indicate a bug in just: {message}\n\
            consider filing an issue: https://github.com/casey/just/issues/new"
@@ -240,10 +241,6 @@ impl Display for CompileError<'_> {
           )
         }
       }
-      ShebangAndScriptAttribute { recipe } => write!(
-        f,
-        "Recipe `{recipe}` has both shebang line and `[script]` attribute"
-      ),
       ShellExpansion { err } => write!(f, "Shell expansion failed: {err}"),
       RequiredParameterFollowsDefaultParameter { parameter } => write!(
         f,
@@ -263,10 +260,9 @@ impl Display for CompileError<'_> {
           List::or_ticked(expected),
         )
       }
-      UnexpectedToken {
-        ref expected,
-        found,
-      } => write!(f, "Expected {}, but found {found}", List::or(expected)),
+      UnexpectedToken { expected, found } => {
+        write!(f, "Expected {}, but found {found}", List::or(expected))
+      }
       UnicodeEscapeCharacter { character } => {
         write!(f, "expected hex digit [0-9A-Fa-f] but found `{character}`")
       }
@@ -295,7 +291,13 @@ impl Display for CompileError<'_> {
       }
       UnknownFunction { function } => write!(f, "Call to unknown function `{function}`"),
       UnknownSetting { setting } => write!(f, "Unknown setting `{setting}`"),
-      UnknownStartOfToken => write!(f, "Unknown start of token:"),
+      UnknownStartOfToken { start } => {
+        write!(f, "Unknown start of token '{start}'")?;
+        if !start.is_ascii_graphic() {
+          write!(f, " (U+{:04X})", *start as u32)?;
+        }
+        Ok(())
+      }
       UnpairedCarriageReturn => write!(f, "Unpaired carriage return"),
       UnterminatedBacktick => write!(f, "Unterminated backtick"),
       UnterminatedInterpolation => write!(f, "Unterminated interpolation"),
