@@ -27,6 +27,9 @@ pub(crate) enum CompileErrorKind<'src> {
     attribute: &'src str,
     first: usize,
   },
+  DuplicateDefault {
+    recipe: &'src str,
+  },
   DuplicateParameter {
     recipe: &'src str,
     parameter: &'src str,
@@ -100,9 +103,6 @@ pub(crate) enum CompileErrorKind<'src> {
   RequiredParameterFollowsDefaultParameter {
     parameter: &'src str,
   },
-  ShebangAndScriptAttribute {
-    recipe: &'src str,
-  },
   ShellExpansion {
     err: shellexpand::LookupError<env::VarError>,
   },
