@@ -185,9 +185,6 @@ pub(crate) enum Error<'src> {
     recipe: &'src str,
     io_error: io::Error,
   },
-  TempfileIo {
-    io_error: io::Error,
-  },
   Unknown {
     recipe: &'src str,
     line_number: Option<usize>,
@@ -501,9 +498,6 @@ impl ColorDisplay for Error<'_> {
         write!(f, "Recipe `{recipe}` could not be run because of an IO error while trying to create a temporary \
                    directory or write a file to that directory: {io_error}")?;
       }
-      TempfileIo { io_error } => {
-        write!(f, "Tempfile I/O error: {io_error}")?;
-      }
       Unknown { recipe, line_number} => {
         if let Some(n) = line_number {
           write!(f, "Recipe `{recipe}` failed on line {n} for an unknown reason")?;
