@@ -23,11 +23,10 @@ impl<'src, 'run> Evaluator<'src, 'run> {
       config,
       dotenv,
       module,
-      scope: parent,
       search,
     };
 
-    let mut scope = context.scope.child();
+    let mut scope = parent.child();
     let mut unknown_overrides = Vec::new();
 
     for (name, value) in overrides {
@@ -191,7 +190,9 @@ impl<'src, 'run> Evaluator<'src, 'run> {
         })
       }
       Expression::Concatenation { lhs, rhs } => {
-        Ok(self.evaluate_expression(lhs)? + &self.evaluate_expression(rhs)?)
+        let lhs = self.evaluate_expression(lhs)?;
+        let rhs = self.evaluate_expression(rhs)?;
+        Ok(lhs + &rhs)
       }
       Expression::Conditional {
         condition,
@@ -204,12 +205,30 @@ impl<'src, 'run> Evaluator<'src, 'run> {
           self.evaluate_expression(otherwise)
         }
       }
+      Expression::FormatString { start, expressions } => {
+        let mut value = start.cooked.clone();
+
+        for (expression, string) in expressions {
+          value.push_str(&self.evaluate_expression(expression)?);
+          value.push_str(&string.cooked);
+        }
+
+        if start.kind.indented {
+          Ok(unindent(&value))
+        } else {
+          Ok(value)
+        }
+      }
       Expression::Group { contents } => self.evaluate_expression(contents),
       Expression::Join { lhs: None, rhs } => Ok("/".to_string() + &self.evaluate_expression(rhs)?),
       Expression::Join {
         lhs: Some(lhs),
         rhs,
-      } => Ok(self.evaluate_expression(lhs)? + "/" + &self.evaluate_expression(rhs)?),
+      } => {
+        let lhs = self.evaluate_expression(lhs)?;
+        let rhs = self.evaluate_expression(rhs)?;
+        Ok(lhs + "/" + &rhs)
+      }
       Expression::Or { lhs, rhs } => {
         let lhs = self.evaluate_expression(lhs)?;
         if !lhs.is_empty() {
@@ -245,6 +264,9 @@ impl<'src, 'run> Evaluator<'src, 'run> {
       ConditionalOperator::RegexMatch => Regex::new(&rhs_value)
         .map_err(|source| Error::RegexCompile { source })?
         .is_match(&lhs_value),
+      ConditionalOperator::RegexMismatch => !Regex::new(&rhs_value)
+        .map_err(|source| Error::RegexCompile { source })?
+        .is_match(&lhs_value),
     };
     Ok(condition)
   }
@@ -264,22 +286,26 @@ impl<'src, 'run> Evaluator<'src, 'run> {
       .module
       .settings
       .shell_command(self.context.config);
-    cmd.arg(command);
-    cmd.args(args);
-    cmd.current_dir(self.context.working_directory());
-    cmd.export(
-      &self.context.module.settings,
-      self.context.dotenv,
-      &self.scope,
-      &self.context.module.unexports,
-    );
-    cmd.stdin(Stdio::inherit());
-    cmd.stderr(if self.context.config.verbosity.quiet() {
-      Stdio::null()
-    } else {
-      Stdio::inherit()
-    });
-    InterruptHandler::guard(|| output(cmd))
+
+    cmd
+      .arg(command)
+      .args(args)
+      .current_dir(self.context.working_directory())
+      .export(
+        &self.context.module.settings,
+        self.context.dotenv,
+        &self.scope,
+        &self.context.module.unexports,
+      )
+      .stdin(Stdio::inherit())
+      .stderr(if self.context.config.verbosity.quiet() {
+        Stdio::null()
+      } else {
+        Stdio::inherit()
+      })
+      .stdout(Stdio::piped());
+
+    cmd.output_guard_stdout()
   }
 
   pub(crate) fn evaluate_line(
@@ -291,7 +317,9 @@ impl<'src, 'run> Evaluator<'src, 'run> {
     for (i, fragment) in line.fragments.iter().enumerate() {
       match fragment {
         Fragment::Text { token } => {
-          let lexeme = token.lexeme().replace("{{{{", "{{");
+          let lexeme = token
+            .lexeme()
+            .replace(Lexer::INTERPOLATION_ESCAPE, Lexer::INTERPOLATION_START);
 
           if i == 0 && continued {
             evaluated += lexeme.trim_start();
@@ -312,8 +340,9 @@ impl<'src, 'run> Evaluator<'src, 'run> {
     is_dependency: bool,
     arguments: &[String],
     parameters: &[Parameter<'src>],
+    scope: &'run Scope<'src, 'run>,
   ) -> RunResult<'src, (Scope<'src, 'run>, Vec<String>)> {
-    let mut evaluator = Self::new(context, is_dependency, context.scope);
+    let mut evaluator = Self::new(context, is_dependency, scope);
 
     let mut positional = Vec::new();
 
