@@ -39,6 +39,7 @@ pub(crate) use {
     execution_context::ExecutionContext,
     executor::Executor,
     expression::Expression,
+    format_string_part::FormatStringPart,
     fragment::Fragment,
     function::Function,
     interpreter::Interpreter,
@@ -83,6 +84,7 @@ pub(crate) use {
     string_delimiter::StringDelimiter,
     string_kind::StringKind,
     string_literal::StringLiteral,
+    string_state::StringState,
     subcommand::Subcommand,
     suggestion::Suggestion,
     table::Table,
@@ -104,7 +106,6 @@ pub(crate) use {
   edit_distance::edit_distance,
   lexiclean::Lexiclean,
   libc::EXIT_FAILURE,
-  once_cell::sync::Lazy,
   rand::seq::IndexedRandom,
   regex::Regex,
   serde::{
@@ -120,20 +121,19 @@ pub(crate) use {
     ffi::OsString,
     fmt::{self, Debug, Display, Formatter},
     fs,
-    io::{self, Read, Seek, Write},
+    io::{self, Write},
     iter::{self, FromIterator},
     mem,
     ops::Deref,
     ops::{Index, Range, RangeInclusive},
     path::{self, Path, PathBuf},
     process::{self, Command, ExitStatus, Stdio},
-    rc::Rc,
     str::{self, Chars},
-    sync::{Mutex, MutexGuard, OnceLock},
-    vec,
+    sync::{Arc, LazyLock, Mutex, MutexGuard, OnceLock},
+    thread, vec,
   },
   strum::{Display, EnumDiscriminants, EnumString, IntoStaticStr},
-  tempfile::{tempfile, TempDir},
+  tempfile::TempDir,
   typed_arena::Arena,
   unicode_width::{UnicodeWidthChar, UnicodeWidthStr},
 };
@@ -217,6 +217,7 @@ mod evaluator;
 mod execution_context;
 mod executor;
 mod expression;
+mod format_string_part;
 mod fragment;
 mod function;
 mod interpreter;
@@ -264,6 +265,7 @@ mod source;
 mod string_delimiter;
 mod string_kind;
 mod string_literal;
+mod string_state;
 mod subcommand;
 mod suggestion;
 mod table;
