@@ -27,7 +27,7 @@ pub(crate) struct Parser<'run, 'src> {
   expected_tokens: BTreeSet<TokenKind>,
   file_depth: u32,
   import_offsets: Vec<usize>,
-  module_namepath: &'run Namepath<'src>,
+  module_namepath: Option<&'run Namepath<'src>>,
   next_token: usize,
   recursion_depth: usize,
   tokens: &'run [Token<'src>],
@@ -40,7 +40,7 @@ impl<'run, 'src> Parser<'run, 'src> {
   pub(crate) fn parse(
     file_depth: u32,
     import_offsets: &[usize],
-    module_namepath: &'run Namepath<'src>,
+    module_namepath: Option<&'run Namepath<'src>>,
     tokens: &'run [Token<'src>],
     working_directory: &'run Path,
   ) -> CompileResult<'src, Ast<'src>> {
@@ -141,7 +141,7 @@ impl<'run, 'src> Parser<'run, 'src> {
     Err(self.internal_error("`Parser::advance()` advanced past end of token stream")?)
   }
 
-  /// Return the next token if it is of kind `expected`, otherwise, return an
+  /// Return next token if it is of kind `expected`, otherwise, return an
   /// unexpected token error
   fn expect(&mut self, expected: TokenKind) -> CompileResult<'src, Token<'src>> {
     if let Some(token) = self.accept(expected)? {
@@ -151,6 +151,18 @@ impl<'run, 'src> Parser<'run, 'src> {
     }
   }
 
+  /// Return the next token if it is any of kinds in `expected`, otherwise,
+  /// return an unexpected token error
+  fn expect_any(&mut self, expected: &[TokenKind]) -> CompileResult<'src, Token<'src>> {
+    for &kind in expected {
+      if let Some(token) = self.accept(kind)? {
+        return Ok(token);
+      }
+    }
+
+    Err(self.unexpected_token()?)
+  }
+
   /// Return an unexpected token error if the next token is not an EOL
   fn expect_eol(&mut self) -> CompileResult<'src> {
     self.accept(Comment)?;
@@ -246,10 +258,10 @@ impl<'run, 'src> Parser<'run, 'src> {
     }
   }
 
-  /// Accept a token of kind `Identifier` and parse into a `Name`
-  fn accept_name(&mut self) -> CompileResult<'src, Option<Name<'src>>> {
+  /// Accept a double-colon separated sequence of identifiers
+  fn accept_namepath(&mut self) -> CompileResult<'src, Option<Namepath<'src>>> {
     if self.next_is(Identifier) {
-      Ok(Some(self.parse_name()?))
+      Ok(Some(self.parse_namepath()?))
     } else {
       Ok(None)
     }
@@ -268,13 +280,13 @@ impl<'run, 'src> Parser<'run, 'src> {
 
   /// Accept a dependency
   fn accept_dependency(&mut self) -> CompileResult<'src, Option<UnresolvedDependency<'src>>> {
-    if let Some(recipe) = self.accept_name()? {
+    if let Some(recipe) = self.accept_namepath()? {
       Ok(Some(UnresolvedDependency {
         arguments: Vec::new(),
         recipe,
       }))
     } else if self.accepted(ParenL)? {
-      let recipe = self.parse_name()?;
+      let recipe = self.parse_namepath()?;
 
       let mut arguments = Vec::new();
 
@@ -282,7 +294,7 @@ impl<'run, 'src> Parser<'run, 'src> {
         arguments.push(self.parse_expression()?);
       }
 
-      Ok(Some(UnresolvedDependency { recipe, arguments }))
+      Ok(Some(UnresolvedDependency { arguments, recipe }))
     } else {
       Ok(None)
     }
@@ -411,7 +423,7 @@ impl<'run, 'src> Parser<'run, 'src> {
             let mut groups = Vec::new();
             for attribute in attributes {
               if let Attribute::Group(group) = attribute {
-                groups.push(group.cooked);
+                groups.push(group);
               }
             }
 
@@ -441,9 +453,9 @@ impl<'run, 'src> Parser<'run, 'src> {
             } else {
               let doc = pop_doc_comment(&mut items, eol_since_last_comment);
               items.push(Item::Recipe(self.parse_recipe(
+                take_attributes(),
                 doc,
                 false,
-                take_attributes(),
               )?));
             }
           }
@@ -451,9 +463,9 @@ impl<'run, 'src> Parser<'run, 'src> {
       } else if self.accepted(At)? {
         let doc = pop_doc_comment(&mut items, eol_since_last_comment);
         items.push(Item::Recipe(self.parse_recipe(
+          take_attributes(),
           doc,
           true,
-          take_attributes(),
         )?));
       } else {
         return Err(self.unexpected_token()?);
@@ -475,6 +487,10 @@ impl<'run, 'src> Parser<'run, 'src> {
 
     Ok(Ast {
       items,
+      module_path: self
+        .module_namepath
+        .map(ToString::to_string)
+        .unwrap_or_default(),
       unstable_features: self.unstable_features,
       warnings: Vec::new(),
       working_directory: self.working_directory.into(),
@@ -485,11 +501,11 @@ impl<'run, 'src> Parser<'run, 'src> {
   fn parse_alias(
     &mut self,
     attributes: AttributeSet<'src>,
-  ) -> CompileResult<'src, Alias<'src, Name<'src>>> {
+  ) -> CompileResult<'src, Alias<'src, Namepath<'src>>> {
     self.presume_keyword(Keyword::Alias)?;
     let name = self.parse_name()?;
     self.presume_any(&[Equals, ColonEquals])?;
-    let target = self.parse_name()?;
+    let target = self.parse_namepath()?;
     self.expect_eol()?;
 
     attributes.ensure_valid_attributes("Alias", *name, &[AttributeDiscriminant::Private])?;
@@ -645,6 +661,29 @@ impl<'run, 'src> Parser<'run, 'src> {
     })
   }
 
+  fn parse_format_string(&mut self) -> CompileResult<'src, Expression<'src>> {
+    self.expect_keyword(Keyword::F)?;
+
+    let (token, start) = self.parse_string_literal_token_in_state(StringState::FormatStart)?;
+
+    let kind = StringKind::from_string_or_backtick(token)?;
+
+    let mut more = token.kind == FormatStringStart;
+
+    let mut expressions = Vec::new();
+
+    while more {
+      let expression = self.parse_expression()?;
+      more = self.next_is(FormatStringContinue);
+      expressions.push((
+        expression,
+        self.parse_string_literal_in_state(StringState::FormatContinue(kind))?,
+      ));
+    }
+
+    Ok(Expression::FormatString { start, expressions })
+  }
+
   // Check if the next tokens are a shell-expanded string, i.e., `x"foo"`.
   //
   // This function skips initial whitespace tokens, but thereafter is
@@ -659,16 +698,38 @@ impl<'run, 'src> Parser<'run, 'src> {
 
     tokens
       .next()
-      .is_some_and(|token| token.kind == Identifier && token.lexeme() == "x")
+      .is_some_and(|token| token.kind == Identifier && token.lexeme() == Keyword::X.lexeme())
       && tokens.next().is_some_and(|token| token.kind == StringToken)
   }
 
+  // Check if the next tokens are a format string, i.e., `f"foo"`.
+  //
+  // This function skips initial whitespace tokens, but thereafter is
+  // whitespace-sensitive, so `f"foo"` is a format string, whereas `f
+  // "foo"` is not.
+  fn next_is_format_string(&self) -> bool {
+    let mut tokens = self
+      .tokens
+      .iter()
+      .skip(self.next_token)
+      .skip_while(|token| token.kind == Whitespace);
+
+    tokens
+      .next()
+      .is_some_and(|token| token.kind == Identifier && token.lexeme() == Keyword::F.lexeme())
+      && tokens
+        .next()
+        .is_some_and(|token| matches!(token.kind, StringToken | FormatStringStart))
+  }
+
   /// Parse a value, e.g. `(bar)`
   fn parse_value(&mut self) -> CompileResult<'src, Expression<'src>> {
     if self.next_is(StringToken) || self.next_is_shell_expanded_string() {
       Ok(Expression::StringLiteral {
         string_literal: self.parse_string_literal()?,
       })
+    } else if self.next_is_format_string() {
+      self.parse_format_string()
     } else if self.next_is(Backtick) {
       let next = self.next()?;
       let kind = StringKind::from_string_or_backtick(next)?;
@@ -723,6 +784,14 @@ impl<'run, 'src> Parser<'run, 'src> {
   /// Parse a string literal, e.g. `"FOO"`, returning the string literal and the string token
   fn parse_string_literal_token(
     &mut self,
+  ) -> CompileResult<'src, (Token<'src>, StringLiteral<'src>)> {
+    self.parse_string_literal_token_in_state(StringState::Normal)
+  }
+
+  /// Parse a string literal, e.g. `"FOO"`, returning the string literal and the string token
+  fn parse_string_literal_token_in_state(
+    &mut self,
+    state: StringState,
   ) -> CompileResult<'src, (Token<'src>, StringLiteral<'src>)> {
     let expand = if self.next_is(Identifier) {
       self.expect_keyword(Keyword::X)?;
@@ -731,24 +800,49 @@ impl<'run, 'src> Parser<'run, 'src> {
       false
     };
 
-    let token = self.expect(StringToken)?;
+    let token = match state {
+      StringState::Normal => self.expect(StringToken)?,
+      StringState::FormatStart => self.expect_any(&[StringToken, FormatStringStart])?,
+      StringState::FormatContinue(_) => {
+        self.expect_any(&[FormatStringContinue, FormatStringEnd])?
+      }
+    };
 
-    let kind = StringKind::from_string_or_backtick(token)?;
+    let kind = match state {
+      StringState::Normal | StringState::FormatStart => StringKind::from_string_or_backtick(token)?,
+      StringState::FormatContinue(kind) => kind,
+    };
 
-    let delimiter_len = kind.delimiter_len();
+    let open = if matches!(token.kind, FormatStringContinue | FormatStringEnd) {
+      Lexer::INTERPOLATION_END.len()
+    } else {
+      kind.delimiter_len()
+    };
 
-    let raw = &token.lexeme()[delimiter_len..token.lexeme().len() - delimiter_len];
+    let close = if matches!(token.kind, FormatStringStart | FormatStringContinue) {
+      Lexer::INTERPOLATION_START.len()
+    } else {
+      kind.delimiter_len()
+    };
 
-    let unindented = if kind.indented() {
+    let raw = &token.lexeme()[open..token.lexeme().len() - close];
+
+    let unindented = if kind.indented() && matches!(token.kind, StringToken) {
       unindent(raw)
     } else {
       raw.to_owned()
     };
 
-    let cooked = if kind.processes_escape_sequences() {
-      Self::cook_string(token, &unindented)?
-    } else {
+    let undelimited = if matches!(state, StringState::Normal) {
       unindented
+    } else {
+      unindented.replace(Lexer::INTERPOLATION_ESCAPE, Lexer::INTERPOLATION_START)
+    };
+
+    let cooked = if kind.processes_escape_sequences() {
+      Self::cook_string(token, &undelimited)?
+    } else {
+      undelimited
     };
 
     let cooked = if expand {
@@ -765,6 +859,23 @@ impl<'run, 'src> Parser<'run, 'src> {
         cooked,
         expand,
         kind,
+        part: match token.kind {
+          FormatStringStart => Some(FormatStringPart::Start),
+          FormatStringContinue => Some(FormatStringPart::Continue),
+          FormatStringEnd => Some(FormatStringPart::End),
+          StringToken => {
+            if matches!(state, StringState::Normal) {
+              None
+            } else {
+              Some(FormatStringPart::Single)
+            }
+          }
+          _ => {
+            return Err(token.error(CompileErrorKind::Internal {
+              message: "unexpected token kind while parsing string literal".into(),
+            }))
+          }
+        },
         raw,
       },
     ))
@@ -774,8 +885,8 @@ impl<'run, 'src> Parser<'run, 'src> {
   fn cook_string(token: Token<'src>, text: &str) -> CompileResult<'src, String> {
     #[derive(PartialEq, Eq)]
     enum State {
-      Initial,
       Backslash,
+      Initial,
       Unicode,
       UnicodeValue { hex: String },
     }
@@ -858,11 +969,33 @@ impl<'run, 'src> Parser<'run, 'src> {
     Ok(string_literal)
   }
 
+  // /// Parse a format string literal, e.g. `"foo{"`, `}bar{`, or `}baz"`
+  fn parse_string_literal_in_state(
+    &mut self,
+    string_state: StringState,
+  ) -> CompileResult<'src, StringLiteral<'src>> {
+    let (_token, string_literal) = self.parse_string_literal_token_in_state(string_state)?;
+    Ok(string_literal)
+  }
+
   /// Parse a name from an identifier token
   fn parse_name(&mut self) -> CompileResult<'src, Name<'src>> {
     self.expect(Identifier).map(Name::from_identifier)
   }
 
+  /// Parse a path of `::` separated names
+  fn parse_namepath(&mut self) -> CompileResult<'src, Namepath<'src>> {
+    let first = self.parse_name()?;
+    let mut path = Namepath::from(first);
+
+    while self.accepted(ColonColon)? {
+      let name = self.parse_name()?;
+      path.push(name);
+    }
+
+    Ok(path)
+  }
+
   /// Parse sequence of comma-separated expressions
   fn parse_sequence(&mut self) -> CompileResult<'src, Vec<Expression<'src>>> {
     self.presume(ParenL)?;
@@ -885,9 +1018,9 @@ impl<'run, 'src> Parser<'run, 'src> {
   /// Parse a recipe
   fn parse_recipe(
     &mut self,
+    attributes: AttributeSet<'src>,
     doc: Option<&'src str>,
     quiet: bool,
-    attributes: AttributeSet<'src>,
   ) -> CompileResult<'src, UnresolvedRecipe<'src>> {
     let name = self.parse_name()?;
 
@@ -950,12 +1083,6 @@ impl<'run, 'src> Parser<'run, 'src> {
     let shebang = body.first().is_some_and(Line::is_shebang);
     let script = attributes.contains(AttributeDiscriminant::Script);
 
-    if shebang && script {
-      return Err(name.error(CompileErrorKind::ShebangAndScriptAttribute {
-        recipe: name.lexeme(),
-      }));
-    }
-
     if attributes.contains(AttributeDiscriminant::WorkingDirectory)
       && attributes.contains(AttributeDiscriminant::NoCd)
     {
@@ -996,7 +1123,7 @@ impl<'run, 'src> Parser<'run, 'src> {
       file_depth: self.file_depth,
       import_offsets: self.import_offsets.clone(),
       name,
-      namepath: self.module_namepath.join(name),
+      namepath: None,
       parameters: positional.into_iter().chain(variadic).collect(),
       priors,
       private,
@@ -1050,7 +1177,7 @@ impl<'run, 'src> Parser<'run, 'src> {
               return Err(self.unexpected_token()?);
             }
           }
-        };
+        }
 
         lines.push(Line { fragments, number });
       }
@@ -1104,6 +1231,7 @@ impl<'run, 'src> Parser<'run, 'src> {
         Some(Setting::AllowDuplicateVariables(self.parse_set_bool()?))
       }
       Keyword::DotenvLoad => Some(Setting::DotenvLoad(self.parse_set_bool()?)),
+      Keyword::DotenvOverride => Some(Setting::DotenvOverride(self.parse_set_bool()?)),
       Keyword::DotenvRequired => Some(Setting::DotenvRequired(self.parse_set_bool()?)),
       Keyword::Export => Some(Setting::Export(self.parse_set_bool()?)),
       Keyword::Fallback => Some(Setting::Fallback(self.parse_set_bool()?)),
@@ -1255,8 +1383,7 @@ mod tests {
   fn test(text: &str, want: Tree) {
     let unindented = unindent(text);
     let tokens = Lexer::test_lex(&unindented).expect("lexing failed");
-    let justfile = Parser::parse(0, &[], &Namepath::default(), &tokens, &PathBuf::new())
-      .expect("parsing failed");
+    let justfile = Parser::parse(0, &[], None, &tokens, &PathBuf::new()).expect("parsing failed");
     let have = justfile.tree();
     if have != want {
       println!("parsed text: {unindented}");
@@ -1284,6 +1411,7 @@ mod tests {
     };
   }
 
+  #[track_caller]
   fn error(
     src: &str,
     offset: usize,
@@ -1294,7 +1422,7 @@ mod tests {
   ) {
     let tokens = Lexer::test_lex(src).expect("Lexing failed in parse test...");
 
-    match Parser::parse(0, &[], &Namepath::default(), &tokens, &PathBuf::new()) {
+    match Parser::parse(0, &[], None, &tokens, &PathBuf::new()) {
       Ok(_) => panic!("Parsing unexpectedly succeeded"),
       Err(have) => {
         let want = CompileError {
@@ -1350,6 +1478,12 @@ mod tests {
     tree: (justfile (alias t test)),
   }
 
+  test! {
+    name: alias_module_path,
+    text: "alias fbb := foo::bar::baz",
+    tree: (justfile (alias fbb (foo bar baz))),
+  }
+
   test! {
     name: single_argument_attribute_shorthand,
     text: "[group: 'foo']\nbar:",
@@ -1610,6 +1744,24 @@ mod tests {
     tree: (justfile (recipe foo (body ("bar")))),
   }
 
+  test! {
+    name: recipe_dependency_module,
+    text: "foo: bar::baz",
+    tree: (justfile (recipe foo (deps (bar baz)))),
+  }
+
+  test! {
+    name: recipe_dependency_parenthesis_module,
+    text: "foo: (bar::baz)",
+    tree: (justfile (recipe foo (deps (bar baz)))),
+  }
+
+  test! {
+    name: recipe_dependency_module_mixed,
+      text: "foo: bar::baz qux",
+    tree: (justfile (recipe foo (deps (bar baz) qux))),
+  }
+
   test! {
     name: recipe_line_multiple,
     text: "foo:\n bar\n baz\n {{\"bob\"}}biz",
@@ -2416,6 +2568,24 @@ mod tests {
     tree: (justfile (assignment foo (assert (if a != b c d) == "abc" "error"))),
   }
 
+  test! {
+    name: format_string_simple,
+    text: "foo := f'abc'",
+    tree: (justfile (assignment foo (format "abc"))),
+  }
+
+  test! {
+    name: format_string_expression,
+    text: "foo := f'foo{{ 'abc' + 'xyz' }}bar'",
+    tree: (justfile (assignment foo (format "foo" (+ "abc" "xyz") "bar"))),
+  }
+
+  test! {
+    name: format_string_complex,
+    text: "foo := f'foo{{ 'abc' + 'xyz' }}bar{{ 'hello' }}goodbye'",
+    tree: (justfile (assignment foo (format "foo" (+ "abc" "xyz") "bar" "hello" "goodbye"))),
+  }
+
   error! {
     name:   alias_syntax_multiple_rhs,
     input:  "alias foo := bar baz",
@@ -2423,7 +2593,7 @@ mod tests {
     line:   0,
     column: 17,
     width:  3,
-    kind:   UnexpectedToken { expected: vec![Comment, Eof, Eol], found: Identifier },
+    kind:   UnexpectedToken { expected: vec![ColonColon, Comment, Eof, Eol], found: Identifier },
   }
 
   error! {
@@ -2436,6 +2606,26 @@ mod tests {
     kind:   UnexpectedToken {expected: vec![Identifier], found:Eol},
   }
 
+  error! {
+    name:   alias_syntax_colon_end,
+    input:  "alias foo := bar::\n",
+    offset: 18,
+    line:   0,
+    column: 18,
+    width:  1,
+    kind:   UnexpectedToken {expected: vec![Identifier], found:Eol},
+  }
+
+  error! {
+    name:   alias_syntax_single_colon,
+    input:  "alias foo := bar:baz",
+    offset: 16,
+    line:   0,
+    column: 16,
+    width:  1,
+    kind:   UnexpectedToken {expected: vec![ColonColon, Comment, Eof, Eol], found:Colon},
+  }
+
   error! {
     name:   missing_colon,
     input:  "a b c\nd e f",
@@ -2493,7 +2683,7 @@ mod tests {
     column:  9,
     width:   1,
     kind:    UnexpectedToken{
-      expected: vec![AmpersandAmpersand, Comment, Eof, Eol, Identifier, ParenL],
+      expected: vec![AmpersandAmpersand, ColonColon, Comment, Eof, Eol, Identifier, ParenL],
       found: Equals
     },
   }
@@ -2531,26 +2721,6 @@ mod tests {
     },
   }
 
-  error! {
-    name:   unclosed_parenthesis_in_interpolation,
-    input:  "a:\n echo {{foo(}}",
-    offset:  15,
-    line:   1,
-    column: 12,
-    width:  2,
-    kind:   UnexpectedToken{
-      expected: vec![
-        Backtick,
-        Identifier,
-        ParenL,
-        ParenR,
-        Slash,
-        StringToken,
-      ],
-      found: InterpolationEnd,
-    },
-  }
-
   error! {
     name:   plus_following_parameter,
     input:  "a b c+:",
