@@ -11,11 +11,13 @@ pub(crate) struct Settings<'src> {
   pub(crate) allow_duplicate_variables: bool,
   pub(crate) dotenv_filename: Option<String>,
   pub(crate) dotenv_load: bool,
+  pub(crate) dotenv_override: bool,
   pub(crate) dotenv_path: Option<PathBuf>,
   pub(crate) dotenv_required: bool,
   pub(crate) export: bool,
   pub(crate) fallback: bool,
   pub(crate) ignore_comments: bool,
+  pub(crate) no_exit_message: bool,
   pub(crate) positional_arguments: bool,
   pub(crate) quiet: bool,
   #[serde(skip)]
@@ -29,11 +31,11 @@ pub(crate) struct Settings<'src> {
 }
 
 impl<'src> Settings<'src> {
-  pub(crate) fn from_setting_iter(iter: impl Iterator<Item = Setting<'src>>) -> Self {
+  pub(crate) fn from_table(sets: Table<'src, Set<'src>>) -> Self {
     let mut settings = Self::default();
 
-    for set in iter {
-      match set {
+    for (_name, set) in sets {
+      match set.value {
         Setting::AllowDuplicateRecipes(allow_duplicate_recipes) => {
           settings.allow_duplicate_recipes = allow_duplicate_recipes;
         }
@@ -49,6 +51,9 @@ impl<'src> Settings<'src> {
         Setting::DotenvPath(path) => {
           settings.dotenv_path = Some(PathBuf::from(path.cooked));
         }
+        Setting::DotenvOverride(dotenv_overrride) => {
+          settings.dotenv_override = dotenv_overrride;
+        }
         Setting::DotenvRequired(dotenv_required) => {
           settings.dotenv_required = dotenv_required;
         }
@@ -61,6 +66,9 @@ impl<'src> Settings<'src> {
         Setting::IgnoreComments(ignore_comments) => {
           settings.ignore_comments = ignore_comments;
         }
+        Setting::NoExitMessage(no_exit_message) => {
+          settings.no_exit_message = no_exit_message;
+        }
         Setting::PositionalArguments(positional_arguments) => {
           settings.positional_arguments = positional_arguments;
         }
@@ -215,16 +223,18 @@ mod tests {
     let settings = Settings {
       shell: Some(Interpreter {
         command: StringLiteral {
-          kind: StringKind::from_token_start("\"").unwrap(),
-          raw: "asdf.exe",
           cooked: "asdf.exe".to_string(),
           expand: false,
+          kind: StringKind::from_token_start("\"").unwrap(),
+          part: None,
+          raw: "asdf.exe",
         },
         arguments: vec![StringLiteral {
-          kind: StringKind::from_token_start("\"").unwrap(),
-          raw: "-nope",
           cooked: "-nope".to_string(),
           expand: false,
+          kind: StringKind::from_token_start("\"").unwrap(),
+          part: None,
+          raw: "-nope",
         }],
       }),
       ..Default::default()
