@@ -14,7 +14,7 @@ struct Assignment<'a> {
   export: bool,
   name: &'a str,
   private: bool,
-  value: &'a str,
+  value: serde_json::Value,
 }
 
 #[derive(Debug, Default, Deserialize, PartialEq, Serialize)]
@@ -42,9 +42,9 @@ struct Module<'a> {
   modules: BTreeMap<&'a str, Module<'a>>,
   recipes: BTreeMap<&'a str, Recipe<'a>>,
   settings: Settings<'a>,
+  source: PathBuf,
   unexports: Vec<&'a str>,
   warnings: Vec<&'a str>,
-  source: PathBuf,
 }
 
 #[derive(Debug, Default, Deserialize, PartialEq, Serialize)]
@@ -79,6 +79,7 @@ struct Settings<'a> {
   allow_duplicate_variables: bool,
   dotenv_filename: Option<&'a str>,
   dotenv_load: bool,
+  dotenv_override: bool,
   dotenv_path: Option<&'a str>,
   dotenv_required: bool,
   export: bool,
@@ -179,7 +180,7 @@ fn assignment() {
         "foo",
         Assignment {
           name: "foo",
-          value: "bar",
+          value: "bar".into(),
           ..default()
         },
       )]
@@ -203,7 +204,7 @@ fn private_assignment() {
           "_foo",
           Assignment {
             name: "_foo",
-            value: "foo",
+            value: "foo".into(),
             private: true,
             ..default()
           },
@@ -212,7 +213,7 @@ fn private_assignment() {
           "bar",
           Assignment {
             name: "bar",
-            value: "bar",
+            value: "bar".into(),
             private: true,
             ..default()
           },
@@ -313,7 +314,7 @@ fn dependency_argument() {
         "x",
         Assignment {
           name: "x",
-          value: "foo",
+          value: "foo".into(),
           ..default()
         },
       )]
@@ -425,7 +426,7 @@ fn duplicate_variables() {
         "x",
         Assignment {
           name: "x",
-          value: "bar",
+          value: "bar".into(),
           ..default()
         },
       )]
@@ -782,6 +783,88 @@ fn attribute() {
   );
 }
 
+#[test]
+fn single_metadata_attribute() {
+  case(
+    "
+      [metadata('example')]
+      foo:
+    ",
+    Module {
+      first: Some("foo"),
+      recipes: [(
+        "foo",
+        Recipe {
+          attributes: [json!({"metadata": ["example"]})].into(),
+          name: "foo",
+          namepath: "foo",
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+}
+
+#[test]
+fn multiple_metadata_attributes() {
+  case(
+    "
+      [metadata('example')]
+      [metadata('sample')]
+      foo:
+    ",
+    Module {
+      first: Some("foo"),
+      recipes: [(
+        "foo",
+        Recipe {
+          attributes: [
+            json!({"metadata": ["example"]}),
+            json!({"metadata": ["sample"]}),
+          ]
+          .into(),
+          name: "foo",
+          namepath: "foo",
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+}
+
+#[test]
+fn multiple_metadata_attributes_with_multiple_arguments() {
+  case(
+    "
+      [metadata('example', 'arg1')]
+      [metadata('sample', 'argument')]
+      foo:
+    ",
+    Module {
+      first: Some("foo"),
+      recipes: [(
+        "foo",
+        Recipe {
+          attributes: [
+            json!({"metadata": ["example", "arg1"]}),
+            json!({"metadata": ["sample", "argument"]}),
+          ]
+          .into(),
+          name: "foo",
+          namepath: "foo",
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+}
+
 #[test]
 fn module() {
   case_with_submodule(
@@ -898,3 +981,65 @@ fn doc_attribute_overrides_comment() {
     },
   );
 }
+
+#[test]
+fn format_string() {
+  case(
+    "
+      foo := f'abc'
+    ",
+    Module {
+      assignments: [(
+        "foo",
+        Assignment {
+          name: "foo",
+          value: json!(["format", "abc"]),
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+  case(
+    "
+      foo := f'abc{{'bar'}}xyz'
+    ",
+    Module {
+      assignments: [(
+        "foo",
+        Assignment {
+          name: "foo",
+          value: json!(["format", "abc", "bar", "xyz"]),
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+  case(
+    "
+      foo := f'abc{{'bar'}}xyz{{'baz' + 'buzz'}}123'
+    ",
+    Module {
+      assignments: [(
+        "foo",
+        Assignment {
+          name: "foo",
+          value: json!([
+            "format",
+            "abc",
+            "bar",
+            "xyz",
+            ["concatenate", "baz", "buzz"],
+            "123"
+          ]),
+          ..default()
+        },
+      )]
+      .into(),
+      ..default()
+    },
+  );
+}
