@@ -1,4 +1,4 @@
-pub(crate) use {
+use {
   crate::{
     assert_stdout::assert_stdout,
     assert_success::assert_success,
@@ -23,12 +23,16 @@ pub(crate) use {
     path::{Path, PathBuf, MAIN_SEPARATOR, MAIN_SEPARATOR_STR},
     process::{Command, Stdio},
     str,
+    time::{Duration, Instant},
   },
   tempfile::TempDir,
   temptree::{temptree, tree, Tree},
   which::which,
 };
 
+#[cfg(not(windows))]
+use std::thread;
+
 fn default<T: Default>() -> T {
   Default::default()
 }
@@ -36,6 +40,7 @@ fn default<T: Default>() -> T {
 #[macro_use]
 mod test;
 
+mod alias;
 mod alias_style;
 mod allow_duplicate_recipes;
 mod allow_duplicate_variables;
@@ -47,6 +52,7 @@ mod assignment;
 mod attributes;
 mod backticks;
 mod byte_order_mark;
+mod ceiling;
 mod changelog;
 mod choose;
 mod command;
@@ -55,7 +61,9 @@ mod conditional;
 mod confirm;
 mod constants;
 mod datetime;
+mod default;
 mod delimiters;
+mod dependencies;
 mod directories;
 mod dotenv;
 mod edit;
@@ -67,6 +75,7 @@ mod explain;
 mod export;
 mod fallback;
 mod format;
+mod format_string;
 mod functions;
 #[cfg(unix)]
 mod global;
@@ -74,8 +83,7 @@ mod groups;
 mod ignore_comments;
 mod imports;
 mod init;
-#[cfg(unix)]
-mod interrupts;
+mod interpolation;
 mod invocation_directory;
 mod json;
 mod line_prefixes;
@@ -91,6 +99,7 @@ mod no_cd;
 mod no_dependencies;
 mod no_exit_message;
 mod os_attributes;
+mod parallel;
 mod parameters;
 mod parser;
 mod positional_arguments;
@@ -102,6 +111,7 @@ mod recursion_limit;
 mod regexes;
 mod request;
 mod run;
+mod scope;
 mod script;
 mod search;
 mod search_arguments;
@@ -110,6 +120,8 @@ mod shebang;
 mod shell;
 mod shell_expansion;
 mod show;
+#[cfg(unix)]
+mod signals;
 mod slash_operator;
 mod string;
 mod subsequents;
